@@ -1015,6 +1015,12 @@ PacketFilter::Action TlsExtensionFilter::FilterExtensions(
   return KEEP;
 }
 
+PacketFilter::Action TlsExtensionOrderCapture::FilterExtension(
+    uint16_t extension_type, const DataBuffer& input, DataBuffer* output) {
+  order.push_back(extension_type);
+  return KEEP;
+}
+
 PacketFilter::Action TlsExtensionCapture::FilterExtension(
     uint16_t extension_type, const DataBuffer& input, DataBuffer* output) {
   if (extension_type == extension_ && (last_ || !captured_)) {
@@ -1199,15 +1205,7 @@ PacketFilter::Action SelectiveRecordDropFilter::FilterRecord(
   return pattern;
 }
 
-PacketFilter::Action TlsClientHelloVersionSetter::FilterHandshake(
-    const HandshakeHeader& header, const DataBuffer& input,
-    DataBuffer* output) {
-  *output = input;
-  output->Write(0, version_, 2);
-  return CHANGE;
-}
-
-PacketFilter::Action TlsServerHelloVersionSetter::FilterHandshake(
+PacketFilter::Action TlsMessageVersionSetter::FilterHandshake(
     const HandshakeHeader& header, const DataBuffer& input,
     DataBuffer* output) {
   *output = input;
@@ -1270,4 +1268,26 @@ PacketFilter::Action ClientHelloPreambleCapture::FilterHandshake(
   return KEEP;
 }
 
+PacketFilter::Action ClientHelloCiphersuiteCapture::FilterHandshake(
+    const HandshakeHeader& header, const DataBuffer& input,
+    DataBuffer* output) {
+  EXPECT_TRUE(header.handshake_type() == kTlsHandshakeClientHello);
+
+  if (captured_) {
+    return KEEP;
+  }
+  captured_ = true;
+
+  TlsParser parser(input);
+  EXPECT_TRUE(parser.Skip(2 + 32));     // Version + Random
+  EXPECT_TRUE(parser.SkipVariable(1));  // Session ID
+  if (is_dtls_agent()) {
+    EXPECT_TRUE(parser.SkipVariable(1));  // Cookie
+  }
+
+  EXPECT_TRUE(parser.ReadVariable(&data_, 2));  // Ciphersuites
+
+  return KEEP;
+}
+
 }  // namespace nss_test
